@@ -1,6 +1,24 @@
 {-# OPTIONS_GHC -fno-warn-dodgy-imports -fno-warn-name-shadowing #-}
 module Sound.Tidal.Carabiner where
 
+{-
+    Carabiner.hs - For syncing with the Link protocol over Carabiner.
+    Copyright (C) 2020, Alex McLean and contributors
+
+    This library is free software: you can redistribute it and/or modify
+    it under the terms of the GNU General Public License as published by
+    the Free Software Foundation, either version 3 of the License, or
+    (at your option) any later version.
+
+    This library is distributed in the hope that it will be useful,
+    but WITHOUT ANY WARRANTY; without even the implied warranty of
+    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+    GNU General Public License for more details.
+
+    You should have received a copy of the GNU General Public License
+    along with this library.  If not, see <http://www.gnu.org/licenses/>.
+-}
+
 import Network.Socket hiding (send, sendTo, recv, recvFrom)
 import Network.Socket.ByteString (send, recv)
 import qualified Data.ByteString.Char8 as B8
@@ -35,7 +53,7 @@ listener tidal bpc latency sock =
                let msg = B8.unpack rMsg
                    (name:_:ws) = words msg
                    pairs = pairs' ws
-                   pairs' (a:b:xs) = (a,b):(pairs' xs)
+                   pairs' (a:b:xs) = (a,b):pairs' xs
                    pairs' _ = []
                act tidal bpc latency name pairs
 
@@ -47,17 +65,17 @@ act tidal bpc latency "status" pairs
        when (and [isJust start, isJust bpm, isJust beat]) $ do
          nowM <- getTime Monotonic
          nowO <- O.time
-         let m = (fromIntegral $ sec nowM) + ((fromIntegral $ nsec nowM)/1000000000)
+         let m = fromIntegral (sec nowM) + (fromIntegral (nsec nowM)/1000000000)
              d = nowO - m
-             start' = ((fromIntegral $ fromJust start) / 1000000)
+             start' = fromIntegral (fromJust start) / 1000000
              startO = start' + d
              -- cyc = toRational $ (fromJust beat) / (fromIntegral bpc)
          tempo <- takeMVar (S.sTempoMV tidal)
          let tempo' = tempo {atTime = startO + latency,
                              atCycle = 0,
-                             cps = ((fromJust bpm) / 60) / (fromIntegral bpc)
+                             cps = (fromJust bpm / 60) / fromIntegral bpc
                             }
-         putMVar (S.sTempoMV tidal) $ tempo'
+         putMVar (S.sTempoMV tidal) tempo'
 act _ _ _ name _ = putStr $ "Unhandled thingie " ++ name
 
 sendMsg :: Socket -> String -> IO ()
