@@ -1,14 +1,35 @@
-{-# LANGUAGE OverloadedStrings, TypeSynonymInstances, FlexibleInstances, CPP #-}
+{-# LANGUAGE OverloadedStrings, FlexibleInstances, CPP, DeriveFunctor #-}
 {-# LANGUAGE LambdaCase #-}
 {-# OPTIONS_GHC -Wall -fno-warn-orphans -fno-warn-unused-do-bind #-}
 
 module Sound.Tidal.ParseBP where
 
-import           Control.Applicative ((<$>), (<*>), pure)
+{-
+    ParseBP.hs - Parser for Tidal's "mini-notation", inspired by
+      Bernard Bel's BP2 (Bol Processor 2) system.
+    Copyright (C) 2020, Alex McLean and contributors
+
+    This library is free software: you can redistribute it and/or modify
+    it under the terms of the GNU General Public License as published by
+    the Free Software Foundation, either version 3 of the License, or
+    (at your option) any later version.
+
+    This library is distributed in the hope that it will be useful,
+    but WITHOUT ANY WARRANTY; without even the implied warranty of
+    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+    GNU General Public License for more details.
+
+    You should have received a copy of the GNU General Public License
+    along with this library.  If not, see <http://www.gnu.org/licenses/>.
+-}
+
+import           Control.Applicative ()
 import qualified Control.Exception as E
+import           Data.Bifunctor (first)
 import           Data.Colour
 import           Data.Colour.Names
 import           Data.Functor.Identity (Identity)
+import           Data.List (intercalate)
 import           Data.Maybe
 import           Data.Ratio
 import           Data.Typeable (Typeable)
@@ -22,6 +43,7 @@ import           Sound.Tidal.Pattern
 import           Sound.Tidal.UI
 import           Sound.Tidal.Core
 import           Sound.Tidal.Chords (chordTable)
+import           Sound.Tidal.Utils (fromRight)
 
 data TidalParseError = TidalParseError {parsecError :: ParseError,
                                         code :: String
@@ -55,7 +77,38 @@ data TPat a = TPat_Atom (Maybe ((Int, Int), (Int, Int))) a
             | TPat_Repeat Int (TPat a)
             | TPat_EnumFromTo (TPat a) (TPat a)
             | TPat_Var String
-            deriving (Show)
+            deriving (Show, Functor)
+
+tShowList :: (Show a) => [TPat a] -> String
+tShowList vs = "[" ++ intercalate "," (map tShow vs) ++ "]"
+
+tShow :: (Show a) => TPat a -> String
+tShow (TPat_Atom _ v) = "pure " ++ show v
+tShow (TPat_Fast t v) = "fast " ++ show t ++ " $ " ++ tShow v
+tShow (TPat_Slow t v) = "slow " ++ show t ++ " $ " ++ tShow v
+-- TODO - should be _degradeByUsing, but needs a simplified version..
+tShow (TPat_DegradeBy _ r v) = "degradeBy " ++ show r ++ " $ " ++ tShow v
+-- TODO - ditto
+tShow (TPat_CycleChoose _ vs) = "cycleChoose " ++ tShowList vs
+
+tShow (TPat_Euclid a b c v) = "doEuclid (" ++ intercalate ") (" (map tShow [a,b,c])  ++ ") $ " ++ tShow v
+tShow (TPat_Stack vs) = "stack " ++ tShowList vs
+
+tShow (TPat_Polyrhythm mSteprate vs) = "stack [" ++ intercalate ", " (map adjust_speed pats) ++ "]"
+  where adjust_speed (sz, pat) = "(fast (" ++ (steprate ++ "/" ++ show sz) ++ ") $ " ++ pat ++ ")"
+        steprate :: String
+        steprate = maybe base_first tShow mSteprate
+        base_first | null pats = "0"
+                   | otherwise = show $ fst $ head pats
+        pats = map steps_tpat vs
+
+tShow (TPat_Seq vs) = snd $ steps_seq vs
+
+tShow TPat_Silence = "silence"
+tShow (TPat_EnumFromTo a b) = "unwrap $ fromTo <$> (" ++ tShow a ++ ") <*> (" ++ tShow b ++ ")"
+tShow (TPat_Var s) = "getControl " ++ s
+tShow a = "can't happen? " ++ show a
+
 
 toPat :: (Parseable a, Enumerable a) => TPat a -> Pattern a
 toPat = \case
@@ -63,10 +116,9 @@ toPat = \case
    TPat_Atom Nothing x -> pure x
    TPat_Fast t x -> fast (toPat t) $ toPat x
    TPat_Slow t x -> slow (toPat t) $ toPat x
-   TPat_DegradeBy seed amt x -> _degradeByUsing (rotL (0.0001 * (fromIntegral seed)) rand) amt $ toPat x
-   TPat_CycleChoose seed xs -> unwrap $ segment 1 $ chooseBy (rotL (0.0001 * (fromIntegral seed)) rand) $ map toPat xs
-   TPat_Euclid n k s thing ->
-      doEuclid (toPat n) (toPat k) (toPat s) (toPat thing)
+   TPat_DegradeBy seed amt x -> _degradeByUsing (rotL (0.0001 * fromIntegral seed) rand) amt $ toPat x
+   TPat_CycleChoose seed xs -> unwrap $ segment 1 $ chooseBy (rotL (0.0001 * fromIntegral seed) rand) $ map toPat xs
+   TPat_Euclid n k s thing -> doEuclid (toPat n) (toPat k) (toPat s) (toPat thing)
    TPat_Stack xs -> stack $ map toPat xs
    TPat_Silence -> silence
    TPat_EnumFromTo a b -> unwrap $ fromTo <$> toPat a <*> toPat b
@@ -75,7 +127,7 @@ toPat = \case
      where adjust_speed (sz, pat) = fast ((/sz) <$> steprate) pat
            pats = map resolve_tpat ps
            steprate :: Pattern Rational
-           steprate = fromMaybe base_first (toPat <$> mSteprate)
+           steprate = (maybe base_first toPat mSteprate)
            base_first | null pats = pure 0
                       | otherwise = pure $ fst $ head pats
    TPat_Seq xs -> snd $ resolve_seq xs
@@ -93,17 +145,25 @@ resolve_seq xs = (total_size, timeCat sized_pats)
 
 resolve_size :: [TPat a] -> [(Rational, TPat a)]
 resolve_size [] = []
-resolve_size ((TPat_Elongate r p):ps) = (r, p):(resolve_size ps)
-resolve_size ((TPat_Repeat n p):ps) = replicate n (1,p) ++ (resolve_size ps)
-resolve_size (p:ps) = (1,p):(resolve_size ps)
+resolve_size ((TPat_Elongate r p):ps) = (r, p):resolve_size ps
+resolve_size ((TPat_Repeat n p):ps) = replicate n (1,p) ++ resolve_size ps
+resolve_size (p:ps) = (1,p):resolve_size ps
 
-{-
-durations :: [TPat a] -> [(Int, TPat a)]
-durations [] = []
-durations (TPat_Elongate n : xs) = (n, TPat_Silence) : durations xs
-durations (a : TPat_Elongate n : xs) = (n+1,a) : durations xs
-durations (a:xs) = (1,a) : durations xs
--}
+
+steps_tpat :: (Show a) => TPat a -> (Rational, String)
+steps_tpat (TPat_Seq xs) = steps_seq xs
+steps_tpat a = (1, tShow a)
+
+steps_seq :: (Show a) => [TPat a] -> (Rational, String)
+steps_seq xs = (total_size, "timeCat [" ++ intercalate "," (map (\(r,s) -> "(" ++ show r ++ ", " ++ s ++ ")") sized_pats) ++ "]")
+  where sized_pats = steps_size xs
+        total_size = sum $ map fst sized_pats
+
+steps_size :: Show a => [TPat a] -> [(Rational, String)]
+steps_size [] = []
+steps_size ((TPat_Elongate r p):ps) = (r, tShow p):steps_size ps
+steps_size ((TPat_Repeat n p):ps) = replicate n (1, tShow p) ++ steps_size ps
+steps_size (p:ps) = (1,tShow p):steps_size ps
 
 parseBP :: (Enumerable a, Parseable a) => String -> Either ParseError (Pattern a)
 parseBP s = toPat <$> parseTPat s
@@ -117,10 +177,13 @@ parseBP_E s = toE parsed
     toE (Right tp) = toPat tp
 
 parseTPat :: Parseable a => String -> Either ParseError (TPat a)
-parseTPat = parseRhythm tPatParser
+parseTPat = runParser (pSequence f' Prelude.<* eof) (0 :: Int) ""
+  where f' = do tPatParser
+             <|> do symbol "~" <?> "rest"
+                    return TPat_Silence
 
 cP :: (Enumerable a, Parseable a) => String -> Pattern a
-cP s = innerJoin $ parseBP_E <$> (_cX_ getS s)
+cP s = innerJoin $ parseBP_E <$> _cX_ getS s
 
 class Parseable a where
   tPatParser :: MyParser (TPat a)
@@ -149,6 +212,15 @@ instance Enumerable Double where
   fromTo = enumFromTo'
   fromThenTo = enumFromThenTo'
 
+instance Parseable Note where
+  tPatParser = pNote
+  doEuclid = euclidOff
+  getControl = cN_
+
+instance Enumerable Note where
+  fromTo = enumFromTo'
+  fromThenTo = enumFromThenTo'
+
 instance Parseable String where
   tPatParser = pVocable
   doEuclid = euclidOff
@@ -179,7 +251,7 @@ instance Enumerable Int where
 instance Parseable Integer where
   tPatParser = pIntegral
   doEuclid = euclidOff
-  getControl = (fmap fromIntegral) . cI_
+  getControl = fmap fromIntegral . cI_
 
 instance Enumerable Integer where
   fromTo = enumFromTo'
@@ -198,8 +270,7 @@ enumFromTo' :: (Ord a, Enum a) => a -> a -> Pattern a
 enumFromTo' a b | a > b = fastFromList $ reverse $ enumFromTo b a
                 | otherwise = fastFromList $ enumFromTo a b
 
-enumFromThenTo'
-  :: (Ord a, Enum a, Num a) => a -> a -> a -> Pattern a
+enumFromThenTo' :: (Ord a, Enum a, Num a) => a -> a -> a -> Pattern a
 enumFromThenTo' a b c | a > c = fastFromList $ reverse $ enumFromThenTo c (c + (a-b)) a
                       | otherwise = fastFromList $ enumFromThenTo a b c
 
@@ -216,9 +287,6 @@ instance Enumerable ColourD where
 instance (Enumerable a, Parseable a) => IsString (Pattern a) where
   fromString = parseBP_E
 
---instance (Parseable a, Pattern p) => IsString (p a) where
---  fromString = p :: String -> p a
-
 lexer :: P.GenTokenParser String u Data.Functor.Identity.Identity
 lexer   = P.makeTokenParser haskellDef
 
@@ -256,52 +324,51 @@ sign  =  do char '-'
          <|> return Positive
 
 intOrFloat :: MyParser Double
-intOrFloat =  do s   <- sign
-                 num <- naturalOrFloat
-                 return (case num of
-                            Right x -> applySign s x
-                            Left  x -> fromIntegral $ applySign s x
-                        )
+intOrFloat = do -- use 'try' to avoid consuming the first '.' in a '..' range.
+                try float
+                <|> fromIntegral <$> integer
 
-parseRhythm :: Parseable a => MyParser (TPat a) -> String -> Either ParseError (TPat a)
-parseRhythm f = runParser (pSequence f' Prelude.<* eof) (0 :: Int) ""
-  where f' = do f
-                <|> do symbol "~" <?> "rest"
-                       return TPat_Silence
+pSequence :: Parseable a => MyParser (TPat a) -> MyParser (TPat a)
+pSequence f = do
+  spaces
+  s <- many $ do
+    a <- pPart f
+    spaces
+    do
+      try $ symbol ".."
+      b <- pPart f
+      return $ TPat_EnumFromTo a b
+      <|> pElongate a
+      <|> pRepeat a
+      <|> return a
+    <|> do
+      symbol "."
+      return TPat_Foot
+  pRand $ resolve_feet s
+  where resolve_feet ps | length ss > 1 = TPat_Seq $ map TPat_Seq ss
+                        | otherwise = TPat_Seq ps
+          where ss = splitFeet ps
+        splitFeet :: [TPat t] -> [[TPat t]]
+        splitFeet [] = []
+        splitFeet pats = foot : splitFeet pats'
+          where (foot, pats') = takeFoot pats
+                takeFoot [] = ([], [])
+                takeFoot (TPat_Foot:pats'') = ([], pats'')
+                takeFoot (pat:pats'') = first (pat:) $ takeFoot pats''
 
-pSequence :: Parseable a => MyParser (TPat a) -> GenParser Char Int (TPat a)
-pSequence f = do spaces -- TODO is this needed?
-                 -- d <- pFast
-                 s <- many $ do a <- pPart f
+pRepeat :: TPat a -> MyParser (TPat a)
+pRepeat a = do es <- many1 $ do char '!'
+                                n <- (subtract 1 . read <$> many1 digit) <|> return 1
                                 spaces
-                                do try $ symbol ".."
-                                   b <- pPart f
-                                   return $ TPat_EnumFromTo a b
-                                 <|> do rs <- many1 $ do oneOf "@_"
-                                                         r <- ((subtract 1) <$> pRatio) <|> return 1
-                                                         spaces
-                                                         return $ r
-                                        return $ TPat_Elongate (1 + sum rs) a
-                                 <|> do es <- many1 $ do char '!'
-                                                         n <- (((subtract 1) . read) <$> many1 digit) <|> return 1
-                                                         spaces
-                                                         return n
-                                        return $ TPat_Repeat (1 + sum es) a
-                                 <|> return a
-                             <|> do symbol "."
-                                    return TPat_Foot
-                 return $ resolve_feet s
-      where resolve_feet ps | length ss > 1 = TPat_Seq $ map TPat_Seq ss
-                            | otherwise = TPat_Seq ps
-              where ss = splitFeet ps
-            splitFeet :: [TPat t] -> [[TPat t]]
-            splitFeet [] = []
-            splitFeet pats = foot : splitFeet pats'
-              where (foot, pats') = takeFoot pats
-                    takeFoot [] = ([], [])
-                    takeFoot (TPat_Foot:pats'') = ([], pats'')
-                    takeFoot (pat:pats'') = (\(a,b) -> (pat:a,b)) $ takeFoot pats''
+                                return n
+               return $ TPat_Repeat (1 + sum es) a
 
+pElongate :: TPat a -> MyParser (TPat a)
+pElongate a = do rs <- many1 $ do oneOf "@_"
+                                  r <- (subtract 1 <$> pRatio) <|> return 1
+                                  spaces
+                                  return r
+                 return $ TPat_Elongate (1 + sum rs) a
 
 pSingle :: MyParser (TPat a) -> MyParser (TPat a)
 pSingle f = f >>= pRand >>= pMult
@@ -312,9 +379,7 @@ pVar = wrapPos $ do char '^'
                     return $ TPat_Var name
 
 pPart :: Parseable a => MyParser (TPat a) -> MyParser (TPat a)
-pPart f = do pt <- (pSingle f <|> pPolyIn f <|> pPolyOut f <|> pVar) >>= pE >>= pRand
-             spaces -- TODO is this needed?
-             return pt
+pPart f = (pSingle f <|> pPolyIn f <|> pPolyOut f <|> pVar) >>= pE >>= pRand
 
 newSeed :: MyParser Int
 newSeed = do seed <- Text.Parsec.Prim.getState
@@ -327,17 +392,14 @@ pPolyIn f = do x <- brackets $ do s <- pSequence f <?> "sequence"
                pMult x
   where stackTail s = do symbol ","
                          ss <- pSequence f `sepBy` symbol ","
-                         spaces -- TODO needed?
                          return $ TPat_Stack (s:ss)
         chooseTail s = do symbol "|"
                           ss <- pSequence f `sepBy` symbol "|"
-                          spaces -- TODO needed?
                           seed <- newSeed
                           return $ TPat_CycleChoose seed (s:ss)
 
 pPolyOut :: Parseable a => MyParser (TPat a) -> MyParser (TPat a)
 pPolyOut f = do ss <- braces (pSequence f `sepBy` symbol ",")
-                spaces -- TODO needed?
                 base <- do char '%'
                            r <- pSequence pRational <?> "rational number"
                            return $ Just r
@@ -345,10 +407,8 @@ pPolyOut f = do ss <- braces (pSequence f `sepBy` symbol ",")
                 pMult $ TPat_Polyrhythm base ss
              <|>
              do ss <- angles (pSequence f `sepBy` symbol ",")
-                spaces -- TODO needed/wanted?
                 pMult $ TPat_Polyrhythm (Just $ TPat_Atom Nothing 1) ss
 
-
 pCharNum :: MyParser Char
 pCharNum = (letter <|> oneOf "0123456789") <?> "letter or number"
 
@@ -367,22 +427,28 @@ wrapPos p = do b <- getPosition
                return $ addPos tpat
 
 pVocable :: MyParser (TPat String)
-pVocable = wrapPos $ (TPat_Atom Nothing) <$> pString
+pVocable = wrapPos $ TPat_Atom Nothing <$> pString
 
 pChar :: MyParser (TPat Char)
-pChar = wrapPos $ (TPat_Atom Nothing) <$> pCharNum
+pChar = wrapPos $ TPat_Atom Nothing <$> pCharNum
 
 pDouble :: MyParser (TPat Double)
-pDouble = wrapPos $ do f <- choice [intOrFloat, parseNote] <?> "float"
-                       do c <- parseChord
-                          return $ TPat_Stack $ map ((TPat_Atom Nothing) . (+f)) c
-                         <|> return (TPat_Atom Nothing f)
-                      <|>
-                         do c <- parseChord
-                            return $ TPat_Stack $ map (TPat_Atom Nothing) c
-                      <|>
-                         do r <- pRatioChar
-                            return $ TPat_Atom Nothing r
+pDouble = wrapPos $ do s <- sign
+                       f <- choice [fromRational <$> pRatio, parseNote] <?> "float"
+                       let v = applySign s f
+                       do TPat_Stack . map (TPat_Atom Nothing . (+ v)) <$> parseChord
+                         <|> return (TPat_Atom Nothing v)
+                       <|>
+                         do TPat_Stack . map (TPat_Atom Nothing) <$> parseChord
+
+pNote :: MyParser (TPat Note)
+pNote = wrapPos $ fmap (fmap Note) $ do s <- sign
+                                        f <- choice [intOrFloat, parseNote] <?> "float"
+                                        let v = applySign s f
+                                        do TPat_Stack . map (TPat_Atom Nothing . (+ v)) <$> parseChord
+                                           <|> return (TPat_Atom Nothing v)
+                                     <|> do TPat_Stack . map (TPat_Atom Nothing) <$> parseChord
+                                     <|> do TPat_Atom Nothing . fromRational <$> pRatio
 
 pBool :: MyParser (TPat Bool)
 pBool = wrapPos $ do oneOf "t1"
@@ -393,34 +459,34 @@ pBool = wrapPos $ do oneOf "t1"
 
 parseIntNote  :: Integral i => MyParser i
 parseIntNote = do s <- sign
-                  i <- choice [integer, parseNote]
-                  return $ applySign s $ fromIntegral i
-
-parseInt :: MyParser Int
-parseInt = do s <- sign
-              i <- integer
-              return $ applySign s $ fromIntegral i
+                  d <- choice [intOrFloat, parseNote]
+                  if isInt d
+                    then return $ applySign s $ round d
+                    else fail "not an integer"
 
 pIntegral :: Integral a => MyParser (TPat a)
 pIntegral = wrapPos $ do i <- parseIntNote
-                         do c <- parseChord
-                            return $ TPat_Stack $ map ((TPat_Atom Nothing) . (+i)) c
+                         do TPat_Stack . map (TPat_Atom Nothing . (+i)) <$> parseChord
                            <|> return (TPat_Atom Nothing i)
                       <|>
-                         do c <- parseChord
-                            return $ TPat_Stack $ map (TPat_Atom Nothing) c
+                         do TPat_Stack . map (TPat_Atom Nothing) <$> parseChord
 
 parseChord :: (Enum a, Num a) => MyParser [a]
 parseChord = do char '\''
                 name <- many1 $ letter <|> digit
                 let chord = fromMaybe [0] $ lookup name chordTable
                 do char '\''
-                   notFollowedBy space <?> "chord range or 'i'"
+                   notFollowedBy space <?> "chord range or 'i' or 'o'"
                    let n = length chord
                    i <- option n (fromIntegral <$> integer)
                    j <- length <$> many (char 'i')
+                   o <- length <$> many (char 'o')
                    let chord' = take i $ drop j $ concatMap (\x -> map (+ x) chord) [0,12..]
-                   return chord'
+                   -- open voiced chords
+                   let chordo' = if o > 0 && n > 2 then
+                                     [ (chord' !! 0 - 12), (chord' !! 2 - 12), (chord' !! 1) ] ++ reverse (take (length chord' - 3) (reverse chord'))
+                                 else chord'
+                   return chordo'
                   <|> return chord
 
 parseNote :: Num a => MyParser a
@@ -446,7 +512,7 @@ parseNote = do n <- notenum
                               ]
 
 fromNote :: Num a => Pattern String -> Pattern a
-fromNote pat = either (const 0) id . runParser parseNote 0 "" <$> pat
+fromNote pat = fromRight 0 . runParser parseNote 0 "" <$> pat
 
 pColour :: MyParser (TPat ColourD)
 pColour = wrapPos $ do name <- many1 letter <?> "colour name"
@@ -490,42 +556,43 @@ pE thing = do (n,k,s) <- parens pair
                         <|> return (TPat_Atom Nothing 0)
                    return (a, b, c)
 
+pRational :: MyParser (TPat Rational)
+pRational = wrapPos $ TPat_Atom Nothing <$> pRatio
+
 pRatio :: MyParser Rational
-pRatio = do s <- sign
-            n <- read <$> many1 digit
-            result <- do char '%'
-                         d <- decimal
-                         return (n%d)
-                      <|>
-                      do char '.'
-                         frac <- many1 digit
-                         -- A hack, but not sure if doing this
-                         -- numerically would be any faster..
-                         return (toRational ((read $ show n ++ "." ++ frac)  :: Double))
-                      <|>
-                      return (n%1)
-            c <- (pRatioChar <|> return 1)
-            return $ applySign s (result * c)
-         <|> pRatioChar
+pRatio = do 
+  s <- sign
+  r <- do n <- try intOrFloat
+          v <- pFraction n <|> return (toRational n)
+          r <- pRatioChar <|> return 1
+          return (v * r)
+       <|>
+       pRatioChar
+  return $ applySign s r
+
+pFraction :: RealFrac a => a -> MyParser Rational
+pFraction n = do
+  char '%'
+  d <- integer
+  if (isInt n)
+    then return ((round n) % d)
+    else fail "fractions need int numerator and denominator"
 
 pRatioChar :: Fractional a => MyParser a
-pRatioChar = do char 'w'
-                return $ 1
-             <|> do char 'h'
-                    return $ 0.5
-             <|> do char 'q'
-                    return $ 0.25
-             <|> do char 'e'
-                    return $ 0.125
-             <|> do char 's'
-                    return $ 0.0625
-             <|> do char 't'
-                    return $ 1/3
-             <|> do char 'f'
-                    return $ 0.2
-             <|> do char 'x'
-                    return $ 1/6
+pRatioChar = pRatioSingleChar 'w' 1
+             <|> pRatioSingleChar 'h' 0.5
+             <|> pRatioSingleChar 'q' 0.25
+             <|> pRatioSingleChar 'e' 0.125
+             <|> pRatioSingleChar 's' 0.0625
+             <|> pRatioSingleChar 't' (1/3)
+             <|> pRatioSingleChar 'f' 0.2
+             <|> pRatioSingleChar 'x' (1/6)
 
-pRational :: MyParser (TPat Rational)
-pRational = wrapPos $ (TPat_Atom Nothing) <$> pRatio
+pRatioSingleChar :: Fractional a => Char -> a -> MyParser a
+pRatioSingleChar c v = try $ do
+  char c
+  notFollowedBy (letter)
+  return v
 
+isInt :: RealFrac a => a -> Bool
+isInt x = x == fromInteger (round x)
\ No newline at end of file
