@@ -2,15 +2,30 @@
 
 module Sound.Tidal.UI where
 
+{-
+    UI.hs - Tidal's main 'user interface' functions, for transforming
+    patterns, building on the Core ones.
+    Copyright (C) 2020, Alex McLean and contributors
+
+    This library is free software: you can redistribute it and/or modify
+    it under the terms of the GNU General Public License as published by
+    the Free Software Foundation, either version 3 of the License, or
+    (at your option) any later version.
+
+    This library is distributed in the hope that it will be useful,
+    but WITHOUT ANY WARRANTY; without even the implied warranty of
+    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+    GNU General Public License for more details.
+
+    You should have received a copy of the GNU General Public License
+    along with this library.  If not, see <http://www.gnu.org/licenses/>.
+-}
+
 import           Prelude hiding ((<*), (*>))
 
 import           Data.Char (digitToInt, isDigit, ord)
 import           Data.Bits (testBit, Bits, xor, shiftL, shiftR)
--- import           System.Random (randoms, mkStdGen)
--- import           Control.Monad.ST
--- import           Control.Monad.Primitive (PrimState, PrimMonad)
--- import qualified Data.Vector as V
--- import           Data.Word (Word32)
+import           Data.Fixed (mod')
 import           Data.Ratio ((%))
 import           Data.List (sort, sortOn, findIndices, elemIndex, groupBy, transpose, intercalate, findIndex)
 import           Data.Maybe (isJust, fromJust, fromMaybe, mapMaybe)
@@ -93,6 +108,17 @@ jux (# ((1024 <~) $ gain rand)) $ sound "sn sn ~ sn" # gain rand
 rand :: Fractional a => Pattern a
 rand = Pattern (\(State a@(Arc s e) _) -> [Event (Context []) Nothing a (realToFrac $ (timeToRand ((e + s)/2) :: Double))])
 
+-- | Boolean rand - a continuous stream of true/false values, with a 50/50 chance.
+brand :: Pattern Bool 
+brand = _brandBy 0.5
+
+-- | Boolean rand with probability as input, e.g. brandBy 0.25 is 25% chance of being true.
+brandBy :: Pattern Double -> Pattern Bool
+brandBy probpat = innerJoin $ (\prob -> _brandBy prob) <$> probpat
+
+_brandBy :: Double -> Pattern Bool
+_brandBy prob = fmap (< prob) rand
+
 {- | Just like `rand` but for whole numbers, `irand n` generates a pattern of (pseudo-) random whole numbers between `0` to `n-1` inclusive. Notably used to pick a random
 samples from a folder:
 
@@ -100,8 +126,11 @@ samples from a folder:
 d1 $ segment 4 $ n (irand 5) # sound "drum"
 @
 -}
-irand :: Num a => Int -> Pattern a
-irand i = fromIntegral . (floor :: Double -> Int) . (* fromIntegral i) <$> rand
+irand :: Num a => Pattern Int -> Pattern a
+irand = (>>= _irand)
+
+_irand :: Num a => Int -> Pattern a
+_irand i = fromIntegral . (floor :: Double -> Int) . (* fromIntegral i) <$> rand
 
 {- | 1D Perlin (smooth) noise, works like rand but smoothly moves between random
 values each cycle. `perlinWith` takes a pattern as the RNG's "input" instead
@@ -113,14 +142,14 @@ will generate a smooth random pattern for the cutoff frequency which will
 repeat every cycle (because the saw does)
 The `perlin` function uses the cycle count as input and can be used much like @rand@.
 -}
-perlinWith :: Pattern Double -> Pattern Double
-perlinWith p = interp <$> (p-pa) <*> (timeToRand <$> pa) <*> (timeToRand <$> pb) where
+perlinWith :: Fractional a => Pattern Double -> Pattern a
+perlinWith p = fmap realToFrac $ (interp) <$> (p-pa) <*> (timeToRand <$> pa) <*> (timeToRand <$> pb) where
   pa = (fromIntegral :: Int -> Double) . floor <$> p
   pb = (fromIntegral :: Int -> Double) . (+1) . floor <$> p
   interp x a b = a + smootherStep x * (b-a)
   smootherStep x = 6.0 * x**5 - 15.0 * x**4 + 10.0 * x**3
 
-perlin :: Pattern Double
+perlin :: Fractional a => Pattern a
 perlin = perlinWith (sig fromRational)
 
 {- `perlin2With` is Perlin noise with a 2-dimensional input. This can be
@@ -203,7 +232,6 @@ d1 $ slow 2 $ degradeBy 0.9 $ sound "[[[feel:5*8,feel*3] feel:3*8], feel*4]"
 @
 
 -}
-
 degradeBy :: Pattern Double -> Pattern a -> Pattern a
 degradeBy = tParam _degradeBy
 
@@ -242,28 +270,46 @@ almostAlways = sometimesBy 0.9
 @
 -}
 sometimesBy :: Pattern Double -> (Pattern a -> Pattern a) -> Pattern a -> Pattern a
-sometimesBy x f p = overlay (degradeBy x p) (unDegradeBy x $ f p)
+sometimesBy x f pat = overlay (degradeBy x pat) (f $ unDegradeBy x pat)
+
+sometimesBy' :: Pattern Double -> (Pattern a -> Pattern a) -> Pattern a -> Pattern a
+sometimesBy' x f pat = overlay (degradeBy x pat) (unDegradeBy x $ f pat)
 
 -- | @sometimes@ is an alias for sometimesBy 0.5.
 sometimes :: (Pattern a -> Pattern a) -> Pattern a -> Pattern a
 sometimes = sometimesBy 0.5
 
+sometimes' :: (Pattern a -> Pattern a) -> Pattern a -> Pattern a
+sometimes' = sometimesBy' 0.5
+
 -- | @often@ is an alias for sometimesBy 0.75.
 often :: (Pattern a -> Pattern a) -> Pattern a -> Pattern a
 often = sometimesBy 0.75
 
+often' :: (Pattern a -> Pattern a) -> Pattern a -> Pattern a
+often' = sometimesBy' 0.75
+
 -- | @rarely@ is an alias for sometimesBy 0.25.
 rarely :: (Pattern a -> Pattern a) -> Pattern a -> Pattern a
 rarely = sometimesBy 0.25
 
+rarely' :: (Pattern a -> Pattern a) -> Pattern a -> Pattern a
+rarely' = sometimesBy' 0.25
+
 -- | @almostNever@ is an alias for sometimesBy 0.1
 almostNever :: (Pattern a -> Pattern a) -> Pattern a -> Pattern a
 almostNever = sometimesBy 0.1
 
+almostNever' :: (Pattern a -> Pattern a) -> Pattern a -> Pattern a
+almostNever' = sometimesBy 0.1
+
 -- | @almostAlways@ is an alias for sometimesBy 0.9
 almostAlways :: (Pattern a -> Pattern a) -> Pattern a -> Pattern a
 almostAlways = sometimesBy 0.9
 
+almostAlways' :: (Pattern a -> Pattern a) -> Pattern a -> Pattern a
+almostAlways' = sometimesBy' 0.9
+
 never :: (Pattern a -> Pattern a) -> Pattern a -> Pattern a
 never = flip const
 
@@ -312,8 +358,6 @@ d1 $ slow 2 $ sound "[[[feel:5*8,feel*3] feel:3*8]?, feel*4]"
 degrade :: Pattern a -> Pattern a
 degrade = _degradeBy 0.5
 
-
-
 {- | (The above means that `brak` is a function from patterns of any type,
 to a pattern of the same type.)
 
@@ -454,7 +498,6 @@ Above, the pattern will have these transforms applied to it, one at a time, per
 
 After `(# speed "0.8")`, the transforms will repeat and start at `density 2` again.
 -}
-
 spread :: (a -> t -> Pattern b) -> [a] -> t -> Pattern b
 spread f xs p = slowcat $ map (`f` p) xs
 
@@ -500,8 +543,6 @@ spreadChoose f vs p = do v <- _segment 1 (choose vs)
 spreadr :: (t -> t1 -> Pattern b) -> [t] -> t1 -> Pattern b
 spreadr = spreadChoose
 
-
-
 {-| Decide whether to apply one or another function depending on the result of a test function that is passed the current cycle as a number.
 
 @
@@ -523,10 +564,14 @@ ifp test f1 f2 p = splitQueries $ p {query = q}
 -- | @wedge t p p'@ combines patterns @p@ and @p'@ by squashing the
 -- @p@ into the portion of each cycle given by @t@, and @p'@ into the
 -- remainer of each cycle.
-wedge :: Time -> Pattern a -> Pattern a -> Pattern a
-wedge 0 _ p' = p'
-wedge 1 p _ = p
-wedge t p p' = overlay (_fastGap (1/t) p) (t `rotR` _fastGap (1/(1-t)) p')
+wedge :: Pattern Time -> Pattern a -> Pattern a -> Pattern a
+wedge pt pa pb = innerJoin $ (\t -> _wedge t pa pb) <$> pt
+
+_wedge :: Time -> Pattern a -> Pattern a -> Pattern a
+_wedge 0 _ p' = p'
+_wedge 1 p _ = p
+_wedge t p p' = overlay (_fastGap (1/t) p) (t `rotR` _fastGap (1/(1-t)) p')
+
 
 {- | @whenmod@ has a similar form and behavior to `every`, but requires an
 additional number. Applies the function to the pattern, when the
@@ -540,8 +585,12 @@ as dense:
 d1 $ whenmod 8 4 (density 2) (sound "bd sn kurt")
 @
 -}
-whenmod :: Int -> Int -> (Pattern a -> Pattern a) -> Pattern a -> Pattern a
-whenmod a b = Sound.Tidal.Core.when (\t -> (t `mod` a) >= b )
+whenmod :: Pattern Time -> Pattern Time -> (Pattern a -> Pattern a) -> Pattern a -> Pattern a
+whenmod a b f pat = innerJoin $ (\a' b' -> _whenmod a' b' f pat) <$> a <*> b
+
+_whenmod :: Time -> Time -> (Pattern a -> Pattern a) -> Pattern a -> Pattern a
+_whenmod a b = whenT (\t -> ((t `mod'` a) >= b ))
+
 
 {- |
 @
@@ -573,17 +622,26 @@ trunc = tParam _trunc
 _trunc :: Time -> Pattern a -> Pattern a
 _trunc t = compress (0, t) . zoomArc (Arc 0 t)
 
-{- | @linger@ is similar to `trunc` but the truncated part of the pattern loops until the end of the cycle
+{- | @linger@ is similar to `trunc` but the truncated part of the pattern loops until the end of the cycle.
 
 @
 d1 $ linger 0.25 $ sound "bd sn*2 cp hh*4 arpy bd*2 cp bd*2"
 @
+
+If you give it a negative number, it will linger on the last part of
+the pattern, instead of the start of it. E.g. to linger on the last
+quarter:
+
+@
+d1 $ linger (-0.25) $ sound "bd sn*2 cp hh*4 arpy bd*2 cp bd*2"
+@
 -}
 linger :: Pattern Time -> Pattern a -> Pattern a
 linger = tParam _linger
 
 _linger :: Time -> Pattern a -> Pattern a
-_linger n p = _fast (1/n) $ zoomArc (Arc 0 n) p
+_linger n p | n < 0 = _fast (1/n) $ zoomArc (Arc (1 + n) 1) p
+            | otherwise = _fast (1/n) $ zoomArc (Arc 0 n) p
 
 {- |
 Use `within` to apply a function to only a part of a pattern. For example, to
@@ -639,7 +697,6 @@ d1 $ sound "[bd bd] hh cp sd"
 @
 
 -}
-
 within' :: (Time, Time) -> (Pattern a -> Pattern a) -> Pattern a -> Pattern a
 within' a@(s, e) f p =
   stack [ filterWhen (\t -> cyclePos t >= s && cyclePos t < e) $ compress a $ f $ zoom a p
@@ -699,28 +756,13 @@ euclid = tParam2 _euclid
 _euclid :: Int -> Int -> Pattern a -> Pattern a
 _euclid n k a = fastcat $ fmap (bool silence a) $ bjorklund (n,k)
 
--- _euclid :: Int -> Int -> Pattern a -> Pattern a
--- _euclid n k p = flip const <$> filterValues (== True) (fastFromList $ bjorklund (n,k)) <*> p
-
 {- | `euclidfull n k pa pb` stacks @e n k pa@ with @einv n k pb@ -}
 euclidFull :: Pattern Int -> Pattern Int -> Pattern a -> Pattern a -> Pattern a
---euclidFull pn pk pa pb = innerJoin $ (\n k -> _euclidFull n k pa pb) <$> pn <*> pk
 euclidFull n k pa pb = stack [ euclid n k pa, euclidInv n k pb ]
 
 _euclidBool :: Int -> Int -> Pattern Bool
 _euclidBool n k = fastFromList $ bjorklund (n,k)
 
-{-_euclidFull :: Int -> Int -> Pattern a -> Pattern a -> Pattern a
-  _euclidFull n k p p' = pickbool <$> _euclidBool n k <*> p <*> p'
-    where pickbool True a _ = a
-          pickbool False _ b = b
--}
-
-
-
--- euclid' :: Pattern Int -> Pattern Int -> Pattern a -> Pattern a
--- euclid' = tParam2 _euclidq'
-
 _euclid' :: Int -> Int -> Pattern a -> Pattern a
 _euclid' n k p = fastcat $ map (\x -> if x then p else silence) (bjorklund (n,k))
 
@@ -766,7 +808,6 @@ euclidInv :: Pattern Int -> Pattern Int -> Pattern a -> Pattern a
 euclidInv = tParam2 _euclidInv
 
 _euclidInv :: Int -> Int -> Pattern a -> Pattern a
---_euclidInv n k p = flip const <$> filterValues (== False) (fastFromList $ bjorklund (n,k)) <*> p
 _euclidInv n k a = fastcat $ fmap (bool a silence) $ bjorklund (n,k)
 
 index :: Real b => b -> Pattern b -> Pattern c -> Pattern c
@@ -918,7 +959,7 @@ discretise = segment
 -- | @randcat ps@: does a @slowcat@ on the list of patterns @ps@ but
 -- randomises the order in which they are played.
 randcat :: [Pattern a] -> Pattern a
-randcat ps = spread' rotL (_segment 1 $ (%1) . fromIntegral <$> (irand (length ps) :: Pattern Int)) (slowcat ps)
+randcat ps = spread' rotL (_segment 1 $ (%1) . fromIntegral <$> (_irand (length ps) :: Pattern Int)) (slowcat ps)
 
 wrandcat :: [(Pattern a, Double)] -> Pattern a
 wrandcat ps = unwrap $ wchooseBy (segment 1 rand) ps
@@ -968,10 +1009,14 @@ d1 $ sound (fit 3 ["bd", "sn", "arpy", "arpy:1", "casio"] "0 [~ 1] 2 1")
 The above fits three samples into the pattern, i.e. for the first cycle this will be `"bd"`, `"sn"` and `"arpy"`, giving the result `"bd [~ sn] arpy sn"` (note that we start counting at zero, so that `0` picks the first value). The following cycle the *next* three values in the list will be picked, i.e. `"arpy:1"`, `"casio"` and `"bd"`, giving the pattern `"arpy:1 [~ casio] bd casio"` (note that the list wraps round here).
 
 -}
-fit :: Int -> [a] -> Pattern Int -> Pattern a
-fit perCycle xs p = (xs !!!) <$> (p {query = map (\e -> fmap (+ pos e) e) . query p})
+_fit :: Int -> [a] -> Pattern Int -> Pattern a
+_fit perCycle xs p = (xs !!!) <$> (p {query = map (\e -> fmap (+ pos e) e) . query p})
   where pos e = perCycle * floor (start $ part e)
 
+fit :: Pattern Int -> [a] -> Pattern Int -> Pattern a
+fit pint xs p = (tParam func) pint (xs,p)
+  where func i (xs',p') = _fit i xs' p'
+
 permstep :: RealFrac b => Int -> [a] -> Pattern b -> Pattern a
 permstep nSteps things p = unwrap $ (\n -> fastFromList $ concatMap (\x -> replicate (fst x) (snd x)) $ zip (ps !! floor (n * fromIntegral (length ps - 1))) things) <$> _segment 1 p
       where ps = permsort (length things) nSteps
@@ -988,10 +1033,10 @@ struct :: Pattern Bool -> Pattern a -> Pattern a
 struct ps pv = filterJust $ (\a b -> if a then Just b else Nothing ) <$> ps <* pv
 
 -- | @substruct a b@: similar to @struct@, but each event in pattern @a@ gets replaced with pattern @b@, compressed to fit the timespan of the event.
-substruct :: Pattern String -> Pattern b -> Pattern b
+substruct :: Pattern Bool -> Pattern b -> Pattern b
 substruct s p = p {query = f}
   where f st =
-          concatMap ((\a' -> queryArc (compressArcTo a' p) a') . fromJust . whole) $ filter isDigital $ (query s st)
+          concatMap ((\a' -> queryArc (compressArcTo a' p) a') . wholeOrPart) $ filter value $ query s st
 
 randArcs :: Int -> Pattern [Arc]
 randArcs n =
@@ -1007,6 +1052,7 @@ randArcs n =
              pairUp' [a, _] = [Arc a 1]
              pairUp' (a:b:xs) = Arc a b: pairUp' (b:xs)
 
+
 -- TODO - what does this do? Something for @stripe@ ..
 randStruct :: Int -> Pattern Int
 randStruct n = splitQueries $ Pattern {query = f}
@@ -1143,18 +1189,9 @@ d1 $ s (mask ("1 ~ 1 ~ 1 1 ~ 1")
   # n (run 8)
 @
 -}
-
 mask :: Pattern Bool -> Pattern a -> Pattern a
 mask b p = const <$> p <* (filterValues id b)
 
-{-
-mask :: Pattern Bool -> Pattern b -> Pattern b
--- TODO - should that be part or whole?
-mask pa pb = pb {query = \st -> concat [filterOns (subArc (arc st) $ part i) (query pb st) | i <- query pa st]}
-     where filterOns Nothing _ = []
-           filterOns (Just a) es = filter (onsetIn a) es
--}
-
 -- | TODO: refactor towards union
 enclosingArc :: [Arc] -> Arc
 enclosingArc [] = Arc 0 1
@@ -1184,7 +1221,7 @@ which uses `chop` to break a single sample into individual pieces, which `fit'`
 
 -}
 fit' :: Pattern Time -> Int -> Pattern Int -> Pattern Int -> Pattern a -> Pattern a
-fit' cyc n from to p = squeezeJoin $ fit n mapMasks to
+fit' cyc n from to p = squeezeJoin $ _fit n mapMasks to
   where mapMasks = [stretch $ mask (const True <$> filterValues (== i) from') p'
                      | i <- [0..n-1]]
         p' = density cyc p
@@ -1196,33 +1233,38 @@ fit' cyc n from to p = squeezeJoin $ fit n mapMasks to
 d1 $ chunk 4 (density 4) $ sound "cp sn arpy [mt lt]"
 @
 -}
-chunk :: Int -> (Pattern b -> Pattern b) -> Pattern b -> Pattern b
-chunk n f p = cat [withinArc (Arc (i % fromIntegral n) ((i+1) % fromIntegral n)) f p | i <- [0 .. fromIntegral n - 1]]
+_chunk :: Int -> (Pattern b -> Pattern b) -> Pattern b -> Pattern b
+_chunk n f p = cat [withinArc (Arc (i % fromIntegral n) ((i+1) % fromIntegral n)) f p | i <- [0 .. fromIntegral n - 1]]
 
-{-
-chunk n f p = do i <- _slow (toRational n) $ run (fromIntegral n)
-                 within (i%(fromIntegral n),(i+)1%(fromIntegral n)) f p
--}
+
+chunk :: Pattern Int -> (Pattern b -> Pattern b) -> Pattern b -> Pattern b
+chunk npat f p  = innerJoin $ (\n -> _chunk n f p) <$> npat
 
 -- deprecated (renamed to chunk)
 runWith :: Int -> (Pattern b -> Pattern b) -> Pattern b -> Pattern b
-runWith = chunk
+runWith = _chunk
 
 {-| @chunk'@ works much the same as `chunk`, but runs from right to left.
 -}
-chunk' :: Integral a => a -> (Pattern b -> Pattern b) -> Pattern b -> Pattern b
-chunk' n f p = do i <- _slow (toRational n) $ rev $ run (fromIntegral n)
-                  withinArc (Arc (i % fromIntegral n) ((i+)1 % fromIntegral n)) f p
+-- this was throwing a parse error when I ran it in tidal whenever I changed the function name..
+_chunk' :: Integral a => a -> (Pattern b -> Pattern b) -> Pattern b -> Pattern b
+_chunk' n f p = do i <- _slow (toRational n) $ rev $ run (fromIntegral n)
+                   withinArc (Arc (i % fromIntegral n) ((i+)1 % fromIntegral n)) f p
 
--- deprecated (renamed to chunk')
-runWith' :: Integral a => a -> (Pattern b -> Pattern b) -> Pattern b -> Pattern b
-runWith' = chunk'
+chunk' :: Integral a1 => Pattern a1 -> (Pattern a2 -> Pattern a2) -> Pattern a2 -> Pattern a2
+chunk' npat f p = innerJoin $ (\n -> _chunk' n f p) <$> npat
+
+_inside :: Time -> (Pattern a1 -> Pattern a) -> Pattern a1 -> Pattern a
+_inside n f p = _fast n $ f (_slow n p)
 
 inside :: Pattern Time -> (Pattern a1 -> Pattern a) -> Pattern a1 -> Pattern a
-inside n f p = density n $ f (slow n p)
+inside np f p = innerJoin $ (\n -> _inside n f p) <$> np
+
+_outside :: Time -> (Pattern a1 -> Pattern a) -> Pattern a1 -> Pattern a
+_outside n = _inside (1/n)
 
 outside :: Pattern Time -> (Pattern a1 -> Pattern a) -> Pattern a1 -> Pattern a
-outside n = inside (1/n)
+outside np f p = innerJoin $ (\n -> _outside n f p) <$> np
 
 loopFirst :: Pattern a -> Pattern a
 loopFirst p = splitQueries $ p {query = f}
@@ -1274,7 +1316,7 @@ cycleChoose = segment 1 . choose
 
 {- | Internal function used by shuffle and scramble -}
 _rearrangeWith :: Pattern Int -> Int -> Pattern a -> Pattern a
-_rearrangeWith ipat n pat = innerJoin $ (\i -> _fast nT $ repeatCycles n $ pats !! i) <$> ipat
+_rearrangeWith ipat n pat = innerJoin $ (\i -> _fast nT $ _repeatCycles n $ pats !! i) <$> ipat
   where
     pats = map (\i -> zoom (fromIntegral i / nT, fromIntegral (i+1) / nT) pat) [0 .. (n-1)]
     nT :: Time
@@ -1301,7 +1343,7 @@ scramble :: Pattern Int -> Pattern a -> Pattern a
 scramble = tParam _scramble
 
 _scramble :: Int -> Pattern a -> Pattern a
-_scramble n = _rearrangeWith (_segment (fromIntegral n) $ irand n) n
+_scramble n = _rearrangeWith (_segment (fromIntegral n) $ _irand n) n
 
 randrun :: Int -> Pattern Int
 randrun 0 = silence
@@ -1315,6 +1357,7 @@ randrun n' =
                 toEv (a',v) = do a'' <- subArc a a'
                                  return $ Event (Context []) (Just a') a'' v
 
+
 ur :: Time -> Pattern String -> [(String, Pattern a)] -> [(String, Pattern a -> Pattern a)] -> Pattern a
 ur t outer_p ps fs = _slow t $ unwrap $ adjust <$> timedValues (getPat . split <$> outer_p)
   where split = wordsBy (==':')
@@ -1430,11 +1473,11 @@ fill p' p = struct (splitQueries $ p {query = q}) p'
 -}
 
 -- Repeats each event @n@ times within its arc
-ply :: Pattern Int -> Pattern a -> Pattern a
+ply :: Pattern Rational -> Pattern a -> Pattern a
 ply = tParam _ply
 
-_ply :: Int -> Pattern a -> Pattern a
-_ply n p = arpeggiate $ stack (replicate n p)
+_ply :: Rational -> Pattern a -> Pattern a
+_ply n pat = squeezeJoin $ (_fast n . pure) <$> pat
 
 -- Like ply, but applies a function each time. The applications are compounded.
 plyWith :: (Ord t, Num t) => Pattern t -> (Pattern a -> Pattern a) -> Pattern a -> Pattern a
@@ -1445,6 +1488,17 @@ _plyWith numPat f p = arpeggiate $ compound numPat
   where compound n | n <= 1 = p
                    | otherwise = overlay p (f $ compound $ n-1)
 
+-- | Syncopates a rhythm, shifting each event halfway into its arc (aka timespan), e.g. @"a b [c d] e"@ becomes the equivalent of @"[~ a] [~ b] [[~ c] [~ d]] [~ e]"@
+press :: Pattern a -> Pattern a
+press = _pressBy 0.5
+
+-- | Like @press@, but allows you to specify the amount in which each event is shifted. @pressBy 0.5@ is the same as @press@, while @pressBy (1/3)@ shifts each event by a third of its arc.
+pressBy :: Pattern Time -> Pattern a -> Pattern a
+pressBy = tParam _pressBy
+
+_pressBy :: Time -> Pattern a -> Pattern a
+_pressBy r pat = squeezeJoin $ (compressTo (r,1) . pure) <$> pat
+
 -- | Uses the first (binary) pattern to switch between the following
 -- two patterns. The resulting structure comes from the source patterns, not the
 -- binary pattern. See also @stitch@.
@@ -1492,17 +1546,17 @@ d1 $ slow 32 $ jux ((# speed "0.5") . rev) $ striateBy 32 (1/16) $ sound "bev"
 @
 -}
 jux
-  :: (Pattern ControlMap -> Pattern ControlMap)
-     -> Pattern ControlMap -> Pattern ControlMap
+  :: (Pattern ValueMap -> Pattern ValueMap)
+     -> Pattern ValueMap -> Pattern ValueMap
 jux = juxBy 1
 juxcut
-  :: (Pattern ControlMap -> Pattern ControlMap)
-     -> Pattern ControlMap -> Pattern ControlMap
+  :: (Pattern ValueMap -> Pattern ValueMap)
+     -> Pattern ValueMap -> Pattern ValueMap
 juxcut f p = stack [p     # P.pan (pure 0) # P.cut (pure (-1)),
                     f $ p # P.pan (pure 1) # P.cut (pure (-2))
                    ]
 
-juxcut' :: [t -> Pattern ControlMap] -> t -> Pattern ControlMap
+juxcut' :: [t -> Pattern ValueMap] -> t -> Pattern ValueMap
 juxcut' fs p = stack $ map (\n -> ((fs !! n) p |+ P.cut (pure $ 1-n)) # P.pan (pure $ fromIntegral n / fromIntegral l)) [0 .. l-1]
   where l = length fs
 
@@ -1529,14 +1583,14 @@ d1 $ stack [
 @
 
 -}
-jux' :: [t -> Pattern ControlMap] -> t -> Pattern ControlMap
+jux' :: [t -> Pattern ValueMap] -> t -> Pattern ValueMap
 jux' fs p = stack $ map (\n -> (fs !! n) p |+ P.pan (pure $ fromIntegral n / fromIntegral l)) [0 .. l-1]
   where l = length fs
 
 -- | Multichannel variant of `jux`, _not sure what it does_
 jux4
-  :: (Pattern ControlMap -> Pattern ControlMap)
-     -> Pattern ControlMap -> Pattern ControlMap
+  :: (Pattern ValueMap -> Pattern ValueMap)
+     -> Pattern ValueMap -> Pattern ValueMap
 jux4 f p = stack [p # P.pan (pure (5/8)), f $ p # P.pan (pure (1/8))]
 
 {- |
@@ -1555,9 +1609,9 @@ and 0.75, rather than 0 and 1.
 -}
 juxBy
   :: Pattern Double
-     -> (Pattern ControlMap -> Pattern ControlMap)
-     -> Pattern ControlMap
-     -> Pattern ControlMap
+     -> (Pattern ValueMap -> Pattern ValueMap)
+     -> Pattern ValueMap
+     -> Pattern ValueMap
 juxBy n f p = stack [p |+ P.pan 0.5 |- P.pan (n/2), f $ p |+ P.pan 0.5 |+ P.pan (n/2)]
 
 pick :: String -> Int -> String
@@ -1605,7 +1659,6 @@ d1 $ jux (iter 4) $ sound "arpy arpy:2*2"
   |+ speed (slow 4 $ range 1 1.5 sine1)
 @
 -}
-
 range :: Num a => Pattern a -> Pattern a -> Pattern a -> Pattern a
 range fromP toP p = (\from to v -> ((v * (to-from)) + from)) <$> fromP *> toP *> p
 
@@ -1647,10 +1700,10 @@ step' ss cs = fastcat $ map f cs
 ghost'' :: Time -> (Pattern a -> Pattern a) -> Pattern a -> Pattern a
 ghost'' a f p = superimpose (((a*2.5) `rotR`) . f) $ superimpose (((a*1.5) `rotR`) . f) p
 
-ghost' :: Time -> Pattern ControlMap -> Pattern ControlMap
+ghost' :: Time -> Pattern ValueMap -> Pattern ValueMap
 ghost' a p = ghost'' a ((|*| P.gain (pure 0.7)) . (|> P.end (pure 0.2)) . (|*| P.speed (pure 1.25))) p
 
-ghost :: Pattern ControlMap -> Pattern ControlMap
+ghost :: Pattern ValueMap -> Pattern ValueMap
 ghost = ghost' 0.125
 
 {- |
@@ -1696,11 +1749,9 @@ _pickF i fs p =  (fs !!! i) p
 -- | @contrast p f f' p'@ splits controlpattern @p'@ in two, applying
 -- the function @f@ to one and @f'@ to the other. This depends on
 -- whether events in it contains values matching with those in @p@.
--- For example in @contrast (n "1") (# crush 3) (# vowel "a") $ n "0 1" # s "bd sn" # speed 3@,
+-- For example in @contrast (# crush 3) (# vowel "a") (n "1") $ n "0 1" # s "bd sn" # speed 3@,
 -- the first event will have the vowel effect applied and the second
 -- will have the crush applied.
-
-
 contrast :: (ControlPattern -> ControlPattern) -> (ControlPattern -> ControlPattern)
             -> ControlPattern -> ControlPattern -> ControlPattern
 contrast = contrastBy (==)
@@ -1727,6 +1778,7 @@ contrastRange
 contrastRange = contrastBy f
       where f (VI s, VI e) (VI v) = v >= s && v <= e
             f (VF s, VF e) (VF v) = v >= s && v <= e
+            f (VN s, VN e) (VN v) = v >= s && v <= e
             f (VS s, VS e) (VS v) = v == s && v == e
             f _ _ = False
 
@@ -1739,22 +1791,32 @@ fix f = contrast f id
 unfix :: (ControlPattern -> ControlPattern) -> ControlPattern -> ControlPattern -> ControlPattern
 unfix = contrast id
 
-fixRange :: (ControlPattern -> Pattern ControlMap)
+fixRange :: (ControlPattern -> Pattern ValueMap)
             -> Pattern (Map.Map String (Value, Value))
             -> ControlPattern
-            -> Pattern ControlMap
+            -> ControlPattern
 fixRange f = contrastRange f id
 
-unfixRange :: (ControlPattern -> Pattern ControlMap)
+unfixRange :: (ControlPattern -> Pattern ValueMap)
               -> Pattern (Map.Map String (Value, Value))
               -> ControlPattern
-              -> Pattern ControlMap
+              -> ControlPattern
 unfixRange = contrastRange id
 
 -- | limit values in a Pattern (or other Functor) to n equally spaced
 -- divisions of 1.
 quantise :: (Functor f, RealFrac b) => b -> f b -> f b
-quantise n = fmap ((/n) . (fromIntegral :: RealFrac b => Int -> b) . floor . (*n))
+quantise n = fmap ((/n) . (fromIntegral :: RealFrac b => Int -> b) . round . (*n))
+
+-- quantise but with floor
+qfloor :: (Functor f, RealFrac b) => b -> f b -> f b
+qfloor n = fmap ((/n) . (fromIntegral :: RealFrac b => Int -> b) . floor . (*n))
+
+qceiling :: (Functor f, RealFrac b) => b -> f b -> f b
+qceiling n = fmap ((/n) . (fromIntegral :: RealFrac b => Int -> b) . ceiling . (*n))
+
+qround :: (Functor f, RealFrac b) => b -> f b -> f b
+qround = quantise
 
 -- | Inverts all the values in a boolean pattern
 inv :: Functor f => f Bool -> f Bool
@@ -1822,7 +1884,7 @@ swap things p = filterJust $ (`lookup` things) <$> p
   d1 $ note (scale "hexDorian" $ snowball (+) (slow 2 . rev) 8 "0 ~ . -1 . 5 3 4 . ~ -2") # s "gtr"
 -}
 snowball :: Int -> (Pattern a -> Pattern a -> Pattern a) -> (Pattern a -> Pattern a) -> Pattern a -> Pattern a
-snowball depth combinationFunction f pattern = cat $ take depth $ scanl combinationFunction pattern $ iterate f pattern
+snowball depth combinationFunction f pattern = cat $ take depth $ scanl combinationFunction pattern $ drop 1 $ iterate f pattern
 
 {- @soak@ |
     applies a function to a pattern and cats the resulting pattern,
@@ -1851,9 +1913,11 @@ deconstruct n p = intercalate " " $ map showStep $ toList p
   slices a pattern `pat` into `n` pieces, then uses the `ipat` pattern of integers to index into those slices.
   So `bite 4 "0 2*2" (run 8)` is the same as `"[0 1] [4 5]*2"`.
 -}
+bite :: Pattern Int -> Pattern Int -> Pattern a -> Pattern a
+bite npat ipat pat = innerJoin $ (\n -> _bite n ipat pat) <$> npat
 
-bite :: Int -> Pattern Int -> Pattern a -> Pattern a
-bite n ipat pat = squeezeJoin $ zoompat <$> ipat
+_bite :: Int -> Pattern Int -> Pattern a -> Pattern a
+_bite n ipat pat = squeezeJoin $ zoompat <$> ipat
   where zoompat i = zoom (i'/(fromIntegral n), (i'+1)/(fromIntegral n)) pat
            where i' = fromIntegral $ i `mod` n
 
@@ -1863,7 +1927,6 @@ squeeze :: Pattern Int -> [Pattern a] -> Pattern a
 squeeze _ [] = silence
 squeeze ipat pats = squeezeJoin $ (pats !!!) <$> ipat
 
-
 squeezeJoinUp :: Pattern (ControlPattern) -> ControlPattern
 squeezeJoinUp pp = pp {query = q}
   where q st = concatMap (f st) (query (filterDigital pp) st)
@@ -1892,11 +1955,29 @@ __binary n num = map (testBit num) $ reverse [0 .. n-1]
 _binary :: Data.Bits.Bits b => Int -> b -> Pattern Bool
 _binary n num = listToPat $ __binary n num
 
-binaryN :: Int -> Pattern Int -> Pattern Bool
-binaryN n p = squeezeJoin $ _binary n <$> p
+_binaryN :: Int -> Pattern Int -> Pattern Bool
+_binaryN n p = squeezeJoin $ _binary n <$> p
+
+binaryN :: Pattern Int -> Pattern Int -> Pattern Bool
+binaryN n p = tParam _binaryN n p
 
 binary :: Pattern Int -> Pattern Bool
 binary = binaryN 8
 
 ascii :: Pattern String -> Pattern Bool
 ascii p = squeezeJoin $ (listToPat . concatMap (__binary 8 . ord)) <$> p
+
+grain :: Pattern Double -> Pattern Double -> ControlPattern
+grain s w = P.begin b # P.end e
+  where b = s
+        e = s + w
+
+-- | For specifying a boolean pattern according to a list of offsets
+-- (aka inter-onset intervals).  For example `necklace 12 [4,2]` is
+-- the same as "t f f f t f t f f f t f". That is, 12 steps per cycle,
+-- with true values alternating between every 4 and every 2 steps.
+necklace :: Rational -> [Int] -> Pattern Bool
+necklace perCycle xs = _slow ((toRational $ sum xs) / perCycle) $ listToPat $ list xs
+  where list :: [Int] -> [Bool]
+        list [] = []
+        list (x:xs') = (True:(replicate (x-1) False)) ++ list xs'
