@@ -1,5 +1,23 @@
 module Sound.Tidal.Utils where
 
+{-
+    Utils.hs - A library of handy Haskell utility functions
+    Copyright (C) 2020, Alex McLean and contributors
+
+    This library is free software: you can redistribute it and/or modify
+    it under the terms of the GNU General Public License as published by
+    the Free Software Foundation, either version 3 of the License, or
+    (at your option) any later version.
+
+    This library is distributed in the hope that it will be useful,
+    but WITHOUT ANY WARRANTY; without even the implied warranty of
+    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+    GNU General Public License for more details.
+
+    You should have received a copy of the GNU General Public License
+    along with this library.  If not, see <http://www.gnu.org/licenses/>.
+-}
+
 import Data.List (delete)
 import System.IO (hPutStrLn, stderr)
 
@@ -75,23 +93,11 @@ wordsBy p s = case dropWhile p s of
    s':rest -> (s':w) : wordsBy p (drop 1 s'')
           where (w, s'') = break p rest
 
--- A hack to add to the source code context for mini-notation, so
--- events know where they are within a whole tidal pattern
-deltaMini :: String -> String
-deltaMini = outside 0 0
-  where outside :: Int -> Int -> String -> String
-        outside _ _ [] = []
-        outside column line ('"':xs) = ("(deltaContext "
-                                         ++ show column
-                                         ++ " "
-                                         ++ show line
-                                         ++ " \""
-                                         ++ inside (column+1) line xs
-                                       )
-        outside _ line ('\n':xs) = '\n':(outside 0 (line+1) xs)
-        outside column line (x:xs) = x:(outside (column+1) line xs)
-        inside :: Int -> Int -> String -> String
-        inside _ _ [] = []
-        inside column line ('"':xs) = '"':')':(outside (column+1) line xs)
-        inside _ line ('\n':xs) = '\n':(inside 0 (line+1) xs)
-        inside column line (x:xs) = x:(inside (column+1) line xs)
+matchMaybe :: Maybe a -> Maybe a -> Maybe a
+matchMaybe Nothing y = y
+matchMaybe x       _ = x
+
+-- Available in Data.Either, but only since 4.10
+fromRight :: b -> Either a b -> b
+fromRight _ (Right b) = b
+fromRight b _         = b
