@@ -21,3 +21,8 @@ run =
           (filterOnsets $ stutWith 4 0.25 (# gain 1) $ sound "bd")
           (filterOnsets $ stut 4 1 0.25 $ sound "bd")
         
+    describe "splice" $ do
+      it "can beatslice" $ do
+        comparePD (Arc 0 1)
+          (splice "4 8" "0 1" $ sound "bev")
+          (begin "0 0.125" # end "0.25 0.25" # speed "0.5 0.25" # sound "bev" # unit "c")
