@@ -249,7 +249,7 @@ run =
 
     describe "controlI" $ do
       it "can retrieve values from state" $
-       (query (pure 3 + cF_ "hello") $ State (Arc 0 1) (Map.singleton "hello" (pure $ VF 0.5)))
+       (query (pure 3 + cF_ "hello") $ State (Arc 0 1) (Map.singleton "hello" (VF 0.5)))
        `shouldBe` [(Event (Context []) (Just $ Arc (0 % 1) (1 % 1)) (Arc (0 % 1) (1 % 1)) 3.5)]
 
     describe "wholeStart" $ do 
@@ -458,24 +458,24 @@ run =
     describe "applyFIS" $ do 
       it "apply Float function when value of type VF" $ do 
         let res = applyFIS (+1) (+1) (++ "1") (VF 1)
-        property $ (VF $ 2.0) === res
+        property $ (VF 2.0) === res
       it "apply Int function when value of type VI" $ do 
         let res = applyFIS (+1) (+1) (++ "1") (VI 1)
-        property $ (VI $ 2) === res
+        property $ (VI 2) === res
       it "apply String function when value of type VS" $ do
         let res = applyFIS (+1) (+1) (++ "1") (VS "1")
-        property $ (VS $ "11") === res 
+        property $ (VS "11") === res 
 
     describe "fNum2" $ do
       it "apply Int function for two Int values" $ do 
         let res = fNum2 (+) (+) (VI 2) (VI 3)
-        property $ (VI $ 5) === res 
+        property $ (VI 5) === res 
       it "apply float function when given two float values" $ do 
         let res = fNum2 (+) (+) (VF 2) (VF 3)
-        property $ (VF $ 5.0) === res 
+        property $ (VF 5.0) === res 
       it "apply float function when one float and one int value given" $ do
         let res = fNum2 (+) (+) (VF 2) (VI 3) 
-        property $ (VF $ 5.0) === res 
+        property $ (VF 5.0) === res 
 
     describe "getI" $ do 
       it "get Just value when Int value is supplied" $ do
@@ -484,7 +484,7 @@ run =
       it "get floored value when float value is supplied" $ do
         let res = getI (VF 3.5)
         property $ (Just 3) === res
-      it "get Nothing if String value is supplied" $ do
+      it "get if String value is supplied" $ do
         let res = getI (VS "3")
         property $ Nothing === res
 
