@@ -1,4 +1,4 @@
-{-# LANGUAGE OverloadedStrings #-}
+{-# LANGUAGE OverloadedStrings, FlexibleInstances, TypeSynonymInstances #-}
 
 module TestUtils where
 
@@ -12,6 +12,28 @@ import Sound.Tidal.Context
 
 import qualified Data.Map.Strict as Map
 
+class TolerantEq a where
+   (~==) :: a -> a -> Bool
+
+instance TolerantEq Double where
+  a ~== b = abs (a - b) < 0.000001
+
+instance TolerantEq Value where
+         (VS a) ~== (VS b) = a == b
+         (VI a) ~== (VI b) = a == b
+         (VR a) ~== (VR b) = a == b
+         (VF a) ~== (VF b) = abs (a - b) < 0.000001
+         _ ~== _ = False
+
+instance TolerantEq a => TolerantEq [a] where
+  as ~== bs = (length as == length bs) && all (uncurry (~==)) (zip as bs)
+
+instance TolerantEq ValueMap where
+  a ~== b = Map.differenceWith (\a' b' -> if a' ~== b' then Nothing else Just a') a b == Map.empty
+
+instance TolerantEq (Event ValueMap) where
+  (Event _ w p x) ~== (Event _ w' p' x') = w == w' && p == p' && x ~== x'
+
 -- | Compare the events of two patterns using the given arc
 compareP :: (Ord a, Show a) => Arc -> Pattern a -> Pattern a -> Property
 compareP a p p' = (sort $ query (stripContext p) $ State a Map.empty) `shouldBe` (sort $ query (stripContext p') $ State a Map.empty)
